@@ -30,11 +30,19 @@
  ** disabled min max scanning when reading from stdin.
  */
 
+/*
+  The official specification of FITS format (which is for more than
+  just visual images) is at
+  ftp://legacy.gsfc.nasa.gov/fits_info/fits_office/fits_standard.pdf
+*/
+
 #include <string.h>
 #include <float.h>
 
+#include "pm_config.h"
 #include "pm_c_util.h"
 #include "mallocvar.h"
+#include "floatcode.h"
 #include "shhopt.h"
 #include "pnm.h"
 
@@ -149,15 +157,124 @@ struct FITS_Header {
 };
 
 
+/* This code deals properly with integers, no matter what the byte order
+   or integer size of the host machine.  We handle sign extension manually to
+   prevent problems with signed/unsigned characters.  We read floating point
+   values properly only when the host architecture conforms to IEEE-754.  If
+   you need to tweak this code for other machines, you might want to get a
+   copy of the FITS documentation from nssdca.gsfc.nasa.gov
+*/
+
+static void
+readFitsChar(FILE *   const ifP,
+             double * const vP) {
+
+    /* 8 bit FITS integers are unsigned */
+
+    int const ich = getc(ifP);
+
+    if (ich == EOF)
+        pm_error("EOF / read error");
+    else
+        *vP = ich;
+}
+
+
+
+static void
+readFitsShort(FILE *   const ifP,
+              double * const vP) {
+
+    int ich;
+    int ival;
+    unsigned char c[8];
+
+    ich = getc(ifP);
+
+    if (ich == EOF)
+        pm_error("EOF / read error");
+
+    c[0] = ich;
+
+    ich = getc(ifP);
+
+    if (ich == EOF)
+        pm_error("EOF / read error");
+
+    c[1] = ich;
+
+    if (c[0] & 0x80)
+        ival = ~0xFFFF | c[0] << 8 | c[1];
+    else
+        ival = c[0] << 8 | c[1];
+
+    *vP = ival;
+}
+
+
+
+static void
+readFitsLong(FILE *   const ifP,
+             double * const vP) {
+
+    unsigned int i;
+    long int lval;
+    unsigned char c[4];
+
+    for (i = 0; i < 4; ++i) {
+        int const ich = getc(ifP);
+        if (ich == EOF)
+            pm_error("EOF / read error");
+        c[i] = ich;
+    }
+
+    if (c[0] & 0x80)
+        lval = ~0xFFFFFFFF | c[0] << 24 | c[1] << 16 | c[2] << 8 | c[3];
+    else
+        lval = c[0] << 24 | c[1] << 16 | c[2] << 8 | c[3] << 0;
+
+    *vP = lval;
+}
+
+
+
+static void
+readFitsFloat(FILE *   const ifP,
+              double * const vP) {
+
+    unsigned int i;
+    pm_bigendFloat bigend;
+
+    for (i = 0; i < 4; ++i) {
+        int const ich = getc(ifP);
+        if (ich == EOF)
+            pm_error("EOF / read error");
+        bigend.bytes[i] = ich;
+    }
+
+    *vP = pm_floatFromBigendFloat(bigend);
+}
+
+
+
+static void
+readFitsDouble(FILE *   const ifP,
+               double * const vP) {
+
+    unsigned int i;
+    pm_bigendDouble bigend;
+
+    for (i = 0; i < 8; ++i) {
+        int const ich = getc(ifP);
+        if (ich == EOF)
+            pm_error("EOF / read error");
+        bigend.bytes[i] = ich;
+    }
+
+    *vP = pm_doubleFromBigendDouble(bigend);
+}
+
 
-/*
- ** This code will deal properly with integers, no matter what the byte order
- ** or integer size of the host machine.  Sign extension is handled manually
- ** to prevent problems with signed/unsigned characters.  Floating point
- ** values will only be read properly when the host architecture is IEEE-754
- ** conformant.  If you need to tweak this code for other machines, you might
- ** want to snag a copy of the FITS documentation from nssdca.gsfc.nasa.gov
- */
 
 static void
 readVal(FILE *   const ifP,
@@ -165,77 +282,25 @@ readVal(FILE *   const ifP,
         double * const vP) {
 
     switch (bitpix) {
-        /* 8 bit FITS integers are unsigned */
-    case 8: {
-        int const ich = getc(ifP);
-        if (ich == EOF)
-            pm_error("EOF / read error");
-        *vP = ich;
-    } break;
+    case 8:
+        readFitsChar(ifP, vP);
+        break;
 
-    case 16: {
-        int ich;
-        int ival;
-        unsigned char c[8];
-
-        ich = getc(ifP);
-        if (ich == EOF)
-            pm_error("EOF / read error");
-        c[0] = ich;
-        ich = getc(ifP);
-        if (ich == EOF)
-            pm_error("EOF / read error");
-        c[1] = ich;
-        if (c[0] & 0x80)
-            ival = ~0xFFFF | c[0] << 8 | c[1];
-        else
-            ival = c[0] << 8 | c[1];
-        *vP = ival;
-    } break;
+    case 16:
+        readFitsShort(ifP, vP);
+        break;
       
-    case 32: {
-        unsigned int i;
-        long int lval;
-        unsigned char c[4];
-
-        for (i = 0; i < 4; ++i) {
-            int const ich = getc(ifP);
-            if (ich == EOF)
-                pm_error("EOF / read error");
-            c[i] = ich;
-        }
-        if (c[0] & 0x80)
-            lval = ~0xFFFFFFFF | c[0] << 24 | c[1] << 16 | c[2] << 8 | c[3];
-        else
-            lval = c[0] << 24 | c[1] << 16 | c[2] << 8 | c[3] << 0;
-        *vP = lval;
-    } break;
+    case 32:
+        readFitsLong(ifP, vP);
+        break;
       
-    case -32: {
-        unsigned int i;
-        unsigned char c[4];
-
-        for (i = 0; i < 4; ++i) {
-            int const ich = getc(ifP);
-            if (ich == EOF)
-                pm_error("EOF / read error");
-            c[i] = ich;
-        }
-        *vP = *((float *)c);
-    } break;
+    case -32:
+        readFitsFloat(ifP, vP);
+        break;
       
-    case -64: {
-        unsigned int i;
-        unsigned char c[8];
-
-        for (i = 0; i < 8; ++i) {
-            int const ich = getc(ifP);
-            if (ich == EOF)
-                pm_error("EOF / read error");
-            c[i] = ich;
-        }
-        *vP = *((double *)c);
-    } break;
+    case -64:
+        readFitsDouble(ifP, vP);
+        break;
       
     default:
         pm_error("Strange bitpix value %d in readVal()", bitpix);
@@ -507,6 +572,13 @@ convertPgmRaster(FILE *             const ifP,
                  double             const datamin,
                  xel **             const xels) {
         
+    /* Note: the FITS specification does not give the association between
+       file position and image position (i.e. is the first pixel in the
+       file the top left, bottom left, etc.).  We use the common sense,
+       popular order of row major, top to bottom, left to right.  This
+       has been the case and accepted since 1989, but in 2008, we discovered
+       that Gimp and ImageMagick do bottom to top.
+    */
     unsigned int image;
 
     pm_message("writing PGM file");
