@@ -10,6 +10,9 @@
 
 #define _XOPEN_SOURCE 500    /* Make sure ftello, fseeko are defined */
 
+#include "netpbm/pm_config.h"
+
+#include <assert.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <stdarg.h>
@@ -18,15 +21,19 @@
 #include <setjmp.h>
 #include <time.h>
 #include <limits.h>
+#if HAVE_FORK
+#include <sys/wait.h>
+#endif
+#include <sys/types.h>
 
-#include "pm_c_util.h"
-#include "mallocvar.h"
-#include "version.h"
+#include "netpbm/pm_c_util.h"
+#include "netpbm/mallocvar.h"
+#include "netpbm/version.h"
+#include "netpbm/nstring.h"
+#include "netpbm/shhopt.h"
 #include "compile.h"
-#include "nstring.h"
-#include "shhopt.h"
 
-#include "pm.h"
+#include "netpbm/pm.h"
 
 /* The following are set by pm_init(), then used by subsequent calls to other
    pm_xxx() functions.
@@ -94,6 +101,86 @@ pm_longjmp(void) {
 
 
 
+void
+pm_fork(int *         const iAmParentP,
+        pid_t *       const childPidP,
+        const char ** const errorP) {
+/*----------------------------------------------------------------------------
+   Same as POSIX fork, except with a nicer interface and works
+   (fails cleanly) on systems that don't have POSIX fork().
+-----------------------------------------------------------------------------*/
+#if HAVE_FORK
+    int rc;
+
+    rc = fork();
+
+    if (rc < 0) {
+        asprintfN(errorP, "Failed to fork a process.  errno=%d (%s)",
+                  errno, strerror(errno));
+    } else {
+        *errorP = NULL;
+
+        if (rc == 0) {
+            *iAmParentP = FALSE;
+        } else {
+            *iAmParentP = TRUE;
+            *childPidP = rc;
+        }
+    }
+#else
+    asprintfN(errorP, "Cannot fork a process, because this system does "
+              "not have POSIX fork()");
+#endif
+}
+
+
+
+void
+pm_waitpid(pid_t         const pid,
+           int *         const statusP,
+           int           const options,
+           pid_t *       const exitedPidP,
+           const char ** const errorP) {
+
+#if HAVE_FORK
+    pid_t rc;
+    rc = waitpid(pid, statusP, options);
+    if (rc == (pid_t)-1) {
+        asprintfN(errorP, "Failed to wait for process exit.  "
+                  "waitpid() errno = %d (%s)",
+                  errno, strerror(errno));
+    } else {
+        *exitedPidP = rc;
+        *errorP = NULL;
+    }
+#else
+    pm_error("INTERNAL ERROR: Attempt to wait for a process we created on "
+             "a system on which we can't create processes");
+#endif
+}
+
+
+
+void
+pm_waitpidSimple(pid_t const pid) {
+
+    int status;
+    pid_t exitedPid;
+    const char * error;
+
+    pm_waitpid(pid, &status, 0, &exitedPid, &error);
+
+    if (error) {
+        pm_errormsg("%s", error);
+        strfree(error);
+        pm_longjmp();
+    } else {
+        assert(exitedPid != 0);
+    }
+}
+
+
+
 void
 pm_setusererrormsgfn(pm_usererrormsgfn * fn) {
 
@@ -190,17 +277,25 @@ pm_error(const char format[], ...) {
 
 
 
+static void *
+mallocz(size_t const size) {
+
+    return malloc(MAX(1, size));
+}
+
+
+
 void *
 pm_allocrow(unsigned int const cols,
             unsigned int const size) {
 
     unsigned char * itrow;
 
-    if (UINT_MAX / cols < size)
+    if (cols != 0 && UINT_MAX / cols < size)
         pm_error("Arithmetic overflow multiplying %u by %u to get the "
                  "size of a row to allocate.", cols, size);
 
-    itrow = malloc(cols * size);
+    itrow = mallocz(cols * size);
     if (itrow == NULL)
         pm_error("out of memory allocating a row");
 
@@ -223,7 +318,7 @@ allocarrayNoHeap(unsigned char ** const rowIndex,
                  unsigned int     const size,
                  const char **    const errorP) {
 
-    if (UINT_MAX / cols < size)
+    if (cols != 0 && UINT_MAX / cols < size)
         asprintfN(errorP,
                   "Arithmetic overflow multiplying %u by %u to get the "
                   "size of a row to allocate.", cols, size);
@@ -234,7 +329,7 @@ allocarrayNoHeap(unsigned char ** const rowIndex,
         *errorP = NULL;
 
         while (rowsDone < rows && !*errorP) {
-            unsigned char * const rowSpace = malloc(cols * size);
+            unsigned char * const rowSpace = mallocz(cols * size);
             if (rowSpace == NULL)
                 asprintfN(errorP,
                           "Unable to allocate a %u-column by %u byte row",
@@ -259,11 +354,11 @@ allocRowHeap(unsigned int const cols,
 
     unsigned char * retval;
 
-    if (UINT_MAX / cols / rows < size)
+    if (cols != 0 && rows != 0 && UINT_MAX / cols / rows < size)
         /* Too big even to request the memory ! */
         retval = NULL;
     else
-        retval = malloc(rows * cols * size);
+        retval = mallocz(rows * cols * size);
 
     return retval;
 }
